# gen_test_data_for_secsel.py
#
# J: SecSel テスト用データの生成
# Python script to generate test data for SecSel,
# biodiversity conservation prioritization tool.
#
# 2020-08-21  TAKENAKA, Akio

import random

# J: 多数の小区画(site)が、正方形状に並んでいる。
# J: その中から保全対象とする site を選ぶ。
# J:
# Small sites are arranged in a square.
# Among them, sites to be conserved are selected.

# J: 正方形の一辺の site 数。この数の二乗がsiteの総数
# J:
# Number of sites along a side of the square
# area_size squared gives the total number of sites.
area_size = 20  
                
# J: あらかじめ保護区に設定しておくsite の数
# J:
# Number of pre-reserved sites (0 or more)
n_pre_reserved = 5  

# J: 保全対象要素(feature)の種類数。あるfeatureの、
# J: ひとつのsite 内に分布する部分を local unit と呼ぶ。
# J:
# Number of biodiversity features. A portion of a feature
# within a site is called a local unit.
n_features   = 8  

# J: 各feature のlocal unit の評価ランクの最大値。
# J: 0 はそのfeatureの不在を表す。
# J:
# Highest rank of local units of features.
# Rank 0 means absence of a feature in a site.
rank_max = 3

# J: 各featureの在確率（ 1 - p_presence が不在確率）
# J:
# Probability of presence of a feature in a site.
p_presence = 0.8  

# J: 保全目標の最小値、最大値（feature ごとに与える）
# J: 最小、最大が同じ値であれば、すべて一定値
# J:
# The minimum and maximu of conservation targets of features.
target_min = 3  
target_max = 5  

# base names of generated files.
file_base_name = "foo"

sites = set()

###
# J: 各featureの各site内のlocal unitの評価データファイル
# J:
# Evaluation data file of local units of features in each site.

file_name = "%s_local_units.txt" % file_base_name
fout = open(file_name, "wt")

# header 

fout.write("site")
for i_feature in range(0, n_features):
    fout.write("\tfeature%02d" % (i_feature + 1))
fout.write("\n")

# evaluation data 

for i in range(0, area_size):
    for j in range(0, area_size):

        site = "%03d%03d" % (i, j)
        sites.add(site)
        fout.write(site)

        for i_feature in  range(0, n_features):

            val = 0  # Inital value (i.e., the feature is absent)

            if random.random() < p_presence:    # When present,
                # determine the value of the local unit
                val = random.randint(1, rank_max) 

            fout.write("\t%d" % val)

        fout.write("\n")

fout.close()


###
# J: コストファイル
# J:
# Data file of cost to conserve each site

file_name = "%s_cost.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("siteID\tcost\n");

for i in range(0, area_size):
    for j in range(0, area_size):
        site = "%03d%03d" % (i, j)
        fout.write("%s\t%.2f\n" % (site, (j + 1) / 10))

fout.close()


###
# J: site間の共有周辺長のデータファイル。
# J: 保全site の散在ペナルティを BOUNTDARY_LEN アルゴリズムで計算する
# J: 場合に参照するデータ。
# J:
# Data file of shared peripheral length between neighbouring sites.
# This data is used when BOUNTDARY_LEN algorithm is used to calculate
# penalty of scattered sites.

file_name = "%s_border.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("siteID1\tsiteID2\tshared_border_len\n");

for i in range(0, area_size):
    for j in range(0, area_size - 1):
        site1 = "%03d%03d" % (i, j)
        site2 = "%03d%03d" % (i, j + 1)
        fout.write("%s\t%s\t1\n" % (site1, site2))

for j in range(0, area_size):
    for i in range(0, area_size - 1):
        site1 = "%03d%03d" % (i, j)
        site2 = "%03d%03d" % (i + 1, j)
        fout.write("%s\t%s\t1\n" % (site1, site2))

fout.close()


###
# J: 各siteの中心座標ののデータファイル。
# J: 保全site の散在ペナルティを DISTANCE_TO_NEAREST アルゴリズムで
# J: 計算する場合に参照するデータ。
# J:
# Data file of shared peripheral length between neighbouring sites.
# This data is used when DISTANCE_TO_NEAREST algorithm is used to calculate
# calculate penalty of scattered sites.

file_name = "%s_xy.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("siteID\tx\ty\n");

for i in range(0, area_size):
    for j in range(0, area_size):

        site = "%03d%03d" % (i, j)
        fout.write("%s\t%.1f\t%.1f\n" % (site, i + 0.5, j + 0.5))

fout.close()


###
# J: 各featureの保全目標のデータファイル
# J:
# Conservation targets of individual features

file_name = "%s_target.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("featureID\ttarget\n");

for i_feature in  range(0, n_features):
    target = random.randint(target_min, target_max)
    fout.write("feature%02d\t%d\n" % (i_feature + 1, target))

fout.close()


###
# J: あらかじめ保全する site のデータファイル
# J:
# Data file of pre-reserved sites.

file_name = "%s_pre_reserved.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("siteID\n");

pre_reserved = random.sample(list(sites), n_pre_reserved)

for site in pre_reserved:
    fout.write("%s\n" % site)

fout.close()


###
# J: 排他的な要素のデータファイル
# J:
# Data file of conflicting features which cannot be
# conserved within a site.

file_name = "%s_conflicts.txt" % file_base_name
fout = open(file_name, "wt")

fout.write("feature01\n")
fout.write("feature02\tfeature03\n")

fout.close()
