# J: SecSel のテストデータによるサイト選択結果を図化する
# J: python スクリプト。
# J: プロットするSecSel 出力ファイル名はコマンドラインで指定する。
# J: ファイルは一回の選択結果のファイルでも、選択回数の集計結果ファイルでもよい。
# J: どちらのファイルかはファイル名で判断する。
# J: 
# Python script to map selected sites by SecSel 
# using artificially generated data
#
# Usage: python map_results.py result_file_name
#
# *result_file_name is the name of a result file of a single selection trial,
# or a summary file of multiple trials.  The type of the result file is
# infered from the file name.
#
# 2020-08-29  by TAKENAKA, Akio

import re
import sys
import os
import matplotlib.pyplot as plt

# J: エリアの大きさ。
# J: 異なる大きさのエリアのマップを描くには以下の行を編集する。
# Size of an area.
# To map sites in a area of different size, edit the following.
x_max = 20
y_max = 20


# J: データファイル名はコマンドラインで与える
# Result file is to be given as a command line parameter
args = sys.argv

# J: コマンドラインにファイル名がない場合、使用法を表示
# Show usage if the file name is not given on the command line
if (len(args) < 2):
    print("Usage: python map_results.py  [result_file_name]\n")
    sys.exit()

file_name = args[1]

# J: 指定したファイルが存在するかチェック
# Chech if the file exists.
if os.path.exists(file_name) == False:
    print(file_name, " not found.")
    sys.exit()

# J: 指定したファイルのタイプ（一回の選択結果か、選択回数の集計か）を取得
# Infer the type of the given result file

summary_flag = False # Will be True if the result file is a summary.

# J: 集計ファイルのベース名は "_sum"で終わる
# Base name of a summary file ends with "_sum"
m = re.search('_sum\.txt', file_name)

if m:  # Summary file
    summary_flag = True

    # J: 集計ファイルなら、色勾配を与えるカラーマップを取得
    # Get colour map
    cmap = 'Reds'
    cm = plt.cm.get_cmap(cmap) 


# J: 集計ファイルでは選択回数の最大値を知る必要があるので、全データを
# J: 以下のリストに入れる。
# Store data to find the maximum selected times when the file is a summary.
site_ids = list()
data = list()


fin = open(file_name, "rt")

fin.readline()  # Skip header line

# J: データを１行ずつ読み込む
# Read each line
while True:

    line = fin.readline()  # read each line
    if not line:
        break

    items = re.split('\s+', line[:-1])  # tab separated, but accepts spaces.

    # J: 各行の最初のデータは site ID
    # The first field is a site ID.
    site = items.pop(0)   
    site_ids.append(site)
    
    # J: 一回の選択結果のファイルの場合、ふたつめのデータは選択順位。あらかじめ保護区
    # J: に設定されている site の場合はすべて 0番目。
    # J: 選択回数の集計ファイルの場合は、ふたつめのデータは選択回数。
    # If the file is a result of a single run, the second field is the order 
    # the site was selected. Pre-reserved sites are marked by 0.
    # If the file is a summary file, the second field is the times the site is selected.

    val = items.pop(0)  
    data.append(int(val))

fin.close()

# J: 集計ファイルの場合、選択回数の最大値を探す
# Find maximum times selected in a summary file
n_max = 0

if summary_flag:
    for v in (data):
        if v > n_max:
            n_max = v


# J: 以下で描画
# Draw a map

fig = plt.figure()
plt.subplots_adjust(left = 0.1, bottom = 0.06, right = 0.99, top = 0.92)
ax = fig.add_subplot(1, 1, 1)

for site, val in zip(site_ids, data):

    # J: サイトの四隅の座標を記録するリスト。要素は x座標、y座標のタプル。
    # Will have four tuples, each consisting of x and y coordinates of corners.
    p = list()

    # J: サイトの座標を site ID から計算する。
    # J: 最初の3桁はX座標、次の3桁はy座標に相当。
    # Get location info from site ID.
    # The first three digits represent the x coordinate
    # and the next three, the y coordinate.
    site_x = int(site[0:3])
    site_y = int(site[3:6])

    p.append((site_x, site_y))
    p.append((site_x + 1, site_y))
    p.append((site_x + 1, site_y + 1))
    p.append((site_x, site_y + 1))

    # J: 一回の選択結果の場合の色指定
    # For a single run
    if summary_flag == False: 
        # Color of selected sites.
        col = "#8888FF"

        # J: 事前に保護区にしていたsite は別の色
        # Color of pre-reserved sites.
        if data == "0":
            col = "#FF8888"

    # J: 選択回数集計ファイルの場合
    # For summary file
    else:  

        idx = val / n_max
        if idx > 1:
            idx = 1.0

        col = cm(idx)

    poly = plt.Polygon(p, facecolor = col)
    ax.add_patch(poly)

ax.set_xlim(0, x_max)
ax.set_ylim(0, y_max)

# J: 指定したファイル名がマップのタイトル
# The title of the map will be the given file name.
plt.title(file_name)

# J: png ファイルを生成する。
# png file is created.
# Base name of the png file is that of the result file.
base_name = re.sub('\.txt', '', file_name)

img_file = "%s.png" % base_name
plt.savefig(img_file)

# J: 画面にマップを表示。
# The image is shown on the display
plt.show()
